/********************************************************************
	TORNADO-E2/6713 Demo Utility.
	FLASH test utility. rev.1A
	(C) MicroLAB Systems, 2007


	Description:
	------------
	This demo tests on-board FLASH memory. For E2/6713 rev.1A board with embedded FLASH memory,
	this must be either of 512K/1M/8M/64Mx8 FLASH memory.


	TORNADO platform:
	-----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

	Copyright:
	----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


	Demo and Signal Parameters:
	---------------------------
	1. On-board configuration switch:
		  - HPI boot mode
		  - FLASH WRITE ENABLE switch SW1-2 must be set to ON
		  - FLASH 64Mx8 BOOT SECTOR WRITE PROTECT switch SW1-3 must be set to OFF
	2. Variable r contains possible error codes:
		(Note, that error codes
			from -1 to -9 are applied to common errors,
			from -10 to -19 are applied to C6xxx on-chip registers errors,
			from -20 to -29 are applied to DSP initialization errors,
			from -30 to -39 are applied to DSP ports configuration errors,
			from -100 to -109 are applied to FLASH PROGRAMMING CONTROL errors,
			from -50 to -99 are applied to user defined errors)
		 0		- no errors;
		-2		- invalid H/W error (not E2/6713 board);
		-3		- H/W error (error writing to the DSP external control registers, etc);
		-4		- incompatible H/W error (the setting is not allowed on this board);
		-10		- error writing to C6713B DSP regs;
		-20		- error during PLL configuration;
		-100	- unsupported FLASH device;
		-101	- FLASH erase error;
		-102	- FLASH write error;
		-103	- FLASH contents does not allow to write this data;
		-104	- invalid FLASH read data during AUTO-SELECT;
		-105	- FLASH writes are disabled by on-board SWITCH SW1-2 or Host Controller;
		-106	- Writes to FLASH 64Mx8 BOOT SECTOR are disabled by on-board SWITCH or Host Controller;
***********************************************************************/

#define __T2E6XXX_SELECT_E6713__			1
#define __T2E6XXX_INCLUDE_FLASH_CONTROL__	1

#include "t2e6xxx_dsp.h"

#define FLASH_512K_TEST_CYCLES		15		//	for 512Kx8 FLASH memory
#define FLASH_1M_TEST_CYCLES		19		//	for 1Mx8 FLASH memory
#define FLASH_8M_TEST_CYCLES		23		//	for 8Mx8 FLASH memory
#define FLASH_64M_TEST_CYCLES		29		//	for 64Mx8 FLASH memory

#define FLASH_CYCLE_LEN				0x100

#define T2E6XXX_FLASH_LEN_ERR		-50		// unsupported FLASH memory length


unsigned FLASH_SADDR[29] 	= { 0x00000, 	0x01000,
								0x02000, 	0x04000,
								0x08000, 	0x08800,
								0x10000, 	0x10400,
								0x18000, 	0x18200,
								0x18100,
								0x20000, 	0x40000,
								0x60000, 	0x70000,	// 512Kx8 test ends here
								0x80000, 	0xa0000,
								0xc0000, 	0xe0000,	// 1Mx8 test ends here
								0x100000,				// 2Mx8 test ends here
								0x200000,				// 4Mx8 test ends here
								0x400000, 	0x700000,	// 8Mx8 test ends here
								0x800000,	0xf00000,
								0x1000000,	0x1fc0000,
								0x2000000,	0x3fa0000	// 64Mx8 test ends here
								};

unsigned FLASH_SDATA[29]	= { 0x00, 0xe9,
								0x3f, 0x55,
								0x25, 0xe3,
								0x8c, 0xa1,
								0x66, 0x99,
								0xbf,
								0xfa, 0x15,
								0x79, 0x59,
								0xe9, 0x15,
								0x7d, 0xf3,
								0x49, 0x5b,
								0x27, 0x09,
								0x7e, 0xe7,
								0x83, 0x38,
								0xcc, 0xd2
								};


unsigned _flash_test_cycles;


void main()
{
	int i, j, r;
	unsigned d, a;
	unsigned manuf_id,dev_id,len,sectors, sector_len;

	// configure DSP PLL with SPREAD_SPECTRUM_CLK not used and initialize DSP
	if ((r = T2E6XXX_init_dsp(0)) != T2E6XXX_OK)
		goto error;

	// enable FLASH writes via FLASH_WR_EN bit of DSP_FLASH_CNTR_RG
	T2E6XXX_SET_FLASH_WR_EN_ON;
	if (!T2E6XXX_FLASH_WR_EN_IS_ON)
	{
		if (T2E6XXX_FLASH_XWRPROT_STAT_IS_ON)
			r = T2E6XXX_FLASH_XWRPROT_ERR;	// FLASH writes are disabled externally!
		else
			r = T2E6XXX_HW_ERR;	// H/W error

		goto error;
	}

	// enable writes to FLASH 64Mx8 BOOT SECTOR via DSP_FLASH_CNTR_RG
	// use this only in case FLASH 64Mx8 chip is installed!!!
	if (T2E6XXX_FLASH_LEN_IS_64MX8)
	{
		T2E6XXX_SET_FLASH64M_BOOT_SECTOR_WR_EN_ON;
		if (!T2E6XXX_FLASH64M_BOOT_SECTOR_WR_EN_IS_ON)
		{
			if (T2E6XXX_FLASH64M_BOOT_SECTOR_XWRPROT_STAT_IS_ON)
				r = T2E6XXX_FLASH64M_BOOT_SECTOR_XWRPROT_ERR;	// Writes to FLASH 64Mx8 BOOT SECTOR are disabled externally!
			else
				r = T2E6XXX_HW_ERR;	// H/W error

			goto error;
		}
	}
	
	// get FLASH info
	if ((r = T2E6XXX_get_flash_info(&manuf_id,&dev_id,&len,&sectors,&sector_len)) != T2E6XXX_OK)
		goto error;

	// set number of FLASH test cycles upon the FLASH length
	switch(dev_id)
	{
		case T2E6XXX_FLASH_DEV_ID_AMD_512KX8:
				_flash_test_cycles = FLASH_512K_TEST_CYCLES;
				break;

		case T2E6XXX_FLASH_DEV_ID_AMD_1MX8:
				_flash_test_cycles = FLASH_1M_TEST_CYCLES;
				break;

		case T2E6XXX_FLASH_DEV_ID_AMD_8MX8:
				_flash_test_cycles = FLASH_8M_TEST_CYCLES;
				break;

		case T2E6XXX_FLASH_DEV_ID_AMD_64MX8:
				_flash_test_cycles = FLASH_64M_TEST_CYCLES;
				break;

			// get here on unsupported FLASH memory length
		default:
				// set user error code for this exceptional situation
				r = T2E6XXX_FLASH_LEN_ERR;
				goto error;
	}

	// erase entire chip with verification
	if ((r = T2E6XXX_erase_flash_chip(T2E6XXX_ON)) != T2E6XXX_OK)
		goto error;

	// now write shifted RAMPS to all predefied FLASH areas
	for (i = 0; i < _flash_test_cycles; i++)
	{
		for (a = FLASH_SADDR[i], j = 0, d = FLASH_SDATA[i];
			j < FLASH_CYCLE_LEN;
			a++,j++,d++
			)
		{
			if ((r = T2E6XXX_write_flash(a,d & T2E6XXX_FLASH_DATA_BITMASK)) != T2E6XXX_OK)
				goto error;
		}
	}

	// now verifies written data
	for (i = 0; i < _flash_test_cycles; i++)
	{
		for (a = FLASH_SADDR[i], j = 0, d = FLASH_SDATA[i];
			j < FLASH_CYCLE_LEN;
			a++,j++,d++)
		{
			if ((r = T2E6XXX_read_flash(a)) != (d & T2E6XXX_FLASH_DATA_BITMASK))
				goto error;
		}
	}

	// finally erase entire chip using sector erase
	for (i = 0; i < sectors; i++)
	{
		if ((r = T2E6XXX_erase_flash_sector(i*sector_len,T2E6XXX_ON)) != T2E6XXX_OK)
			goto error;
	}

halt_ok:
	// TRAP here in case programming & verification is OK
	while(1);

error:
	// TRAP here in case of any error
	// variable 'r' contains returned error code
	while(1);
}

