/****************************************************
	TORNADO-E2/6713 Demo Utility.
	I2C-0 port test. rev.1A
	(C) MicroLAB Systems, 2007


	Description:
	------------
	This demo tests DSP I2C-0 port. 
	This demo configures I2C-0 port as master, then transmits data to and receives from
	 serial EEPROM (AT24C08) at 400 kHz.
	This demo uses two passes to check if all data transmit and receive correctly.
	User must connect I2C0 port on E2/6713 board and serial EEPROM AT24C08 before run this demo.


	TORNADO platform:
	-----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

	Copyright:
	----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.

		Note!!!:
		--------
		1. In order to run this demo I2C0 port on E2/6713 board and serial EEPROM AT24C08
			 must be connected!

	Board configuration and demo parameters:
	----------------------------------------
	1. On-board bootmode configuration switch: HPI
	2. Variable r contains possible error codes:
		(Note, that error codes
			from -1 to -9 are applied to common errors,
			from -10 to -19 are applied to C6xxx on-chip registers errors,
			from -20 to -29 are applied to DSP initialization errors,
			from -30 to -39 are applied to DSP ports configuration errors,
			from -100 to -109 are applied to FLASH PROGRAMMING CONTROL errors,
			from -50 to -99 are applied to user defined errors)
		 0	- no errors;
		-2	- invalid H/W error (not E2/6713 board);
		-3	- H/W error (error writing to the DSP external control registers, etc);
		-4	- incompatible H/W error (the setting is not allowed on this board);
		-10	- error writing to C6713B DSP regs;
		-20	- error during PLL configuration;
		-50...-55	- errors occurred during this test;
*****************************************************/

#define __T2E6XXX_SELECT_E6713__			1

#include "t2e6xxx_dsp.h"

#define T2E6XXX_I2C0_ERR			-50		// I2C0 port is not correctly operated
#define T2E6XXX_I2C0_AL_ERR			-51		// I2C0 port arbitratin lost error
#define T2E6XXX_I2C0_NACK_ERR		-52		// I2C0 port no-acknowledgement error
#define T2E6XXX_I2C0_ARDY_ERR		-53		// I2C0 port ARDY error
#define T2E6XXX_I2C0_RRDY_ERR		-54		// I2C0 port receiver error
#define T2E6XXX_I2C0_XRDY_ERR		-55		// I2C0 port transmitter error

#define WORD_ADDRESS				0x10			// EEPROM word address (for AT24C08 may be from 0x00 to 0xf0)
#define DATA_LEN					16				// Number of data words to be transmitted
													// (it's a max page write length for AT24C08)
#define MSG_LEN						DATA_LEN + 1	// Number of data words to be transmitted
													//  + EEPROM word address

unsigned OUT_DATA[MSG_LEN]		= { WORD_ADDRESS,
									0x01, 0xe9,
									0x3f, 0xf5,
									0xaa, 0xe3,
									0x8c, 0xa1,
									0x11, 0x22,
									0x33, 0x44,
									0x55, 0x66,
									0x77, 0x88
									};

unsigned IN_DATA[DATA_LEN];

int receive_success = 0, // All data words have been received successfully
	received_words = 0, // Number of received data words
	transmit_success = 0, // All data words have been transmitted successfully
	transmit_words = 0, // Number of transmitted data words
	r = 0,
	first_pass = 0; // First data array has been transmitted and received successfully


void main()
{
	int i;
	
	// configure DSP PLL with SPREAD_SPECTRUM_CLK not used and initialize DSP
	if ((r = T2E6XXX_init_dsp(0)) != T2E6XXX_OK)
		goto error;

	// First of all check PIDs of I2C0 port
	if (!C6713_I2C0_PID1_IS_C6713)
	{
		r = T2E6XXX_INVALID_HW_ERR;
		goto error;
	}
	if (!C6713_I2C0_PID2_IS_I2C)
	{
		r = T2E6XXX_INVALID_HW_ERR;
		goto error;
	}

start:
	// This label is used to start second pass with new OUT_DATA array

	// reset I2C0 module
	C6713_SET_I2C0_MDR_IRS_ON;
	// configure I2C0 port as master-transmitter
	// set module clock frequency to 150 MHz
	*C6713_I2C0_PSC_RG = 0;
	// disable all I2C interrupts
	C6713_set_i2c0_ier_rg(0);
	// set master SCL clock frequency to 400 kHz
	*C6713_I2C0_CLKL_RG = 0xb6; // for frequency 100 kHz CLKL = 0x2e8;
	*C6713_I2C0_CLKH_RG = 0xb5; // for frequency 100 kHz CLKL = 0x2e8;
	// configure I2C0 own address to 1000000 and EEPROM own address to 1010000 (depends on EEPROM chip)
	*C6713_I2C0_OAR_RG = 0x40;
	*C6713_I2C0_SAR_RG = 0x50;
	// configure data count register to MSG_LEN data words (this is a size of message + word address for EEPROM)
	*C6713_I2C0_CNT_RG = MSG_LEN;
	// take I2C0 module out of reset condition and configure it as master-transmitter
	// with automatically generated STOP condition (ARDY flag is not used)
	C6713_set_i2c0_mdr_rg(0xe20);
	// wait a little in order to the BB bit will correctly reflect the state of I2C bus
	// (see C6713B_silicon_errata_rev20.pdf)
	for (i = 0; i < 300000; i++);

	// poll the BUS BUSY bit (BB) in the I2C0 Status register
	while (C6713_I2C0_STR_BB_IS_ON);
	// write word address for EEPROM into I2C0 Data Transmit register
	*C6713_I2C0_DXR_RG = OUT_DATA[transmit_words];
	transmit_words++;
	// set START condition for I2C0 module
	C6713_SET_I2C0_MDR_STT_ON;

	while (r == T2E6XXX_OK)
	{
		if (C6713_I2C0_STR_AL_IS_ON)
			// Arbitration is lost: this situation is happened due to
			// incorrect configuration of I2C modules
			r = T2E6XXX_I2C0_AL_ERR;
		else if (C6713_I2C0_STR_NACK_IS_ON)
			// No-acknowledgement condition: this module I2C0 did not received
			// acknowledgement from the slave EEPROM
			r = T2E6XXX_I2C0_NACK_ERR;
		else if (C6713_I2C0_STR_ICXRDY_IS_ON)
		{
			if (!transmit_success)
			{
				// In this case I2C0 module must transmit data otherwise error condition
				if (receive_success)
					r = T2E6XXX_I2C0_ERR;
				else
				{
					// Transmit ready condition: the data transmit register is ready
					// to accept new data
					if (transmit_words == MSG_LEN)
					{
						// in this case all data words have been transmitted
						// and I2C0 module must be reconfigured
						C6713_CLEAR_I2C0_STR_ICXRDY;
						transmit_success = 1;
						// reprogram I2C0 module in order to receive data from EEPROM
						// wait a little in order to set STOP condition on I2C bus
						for (i = 0; i < 300000; i++);
						// reset I2C0 module
						C6713_SET_I2C0_MDR_IRS_ON;
						// configure data count register to 16 data words (this is a size of message)
						*C6713_I2C0_CNT_RG = DATA_LEN;
						// take I2C0 module out of reset condition and configure I2C0 module as master-receiver
						C6713_set_i2c0_mdr_rg(0xc20);
						// wait a little in order to the BB bit will correctly reflect the state of I2C bus
						// (see C6713B_silicon_errata_rev20.pdf)
						for (i = 0; i < 300000; i++);
						// poll the BUS BUSY bit (BB) in the I2C0 Status register
						while (C6713_I2C0_STR_BB_IS_ON);
						// set START condition for I2C0 module
						C6713_SET_I2C0_MDR_STT_ON;
						// start receiving data from EEPROM
					}
					else
					{
						C6713_CLEAR_I2C0_STR_ICXRDY;
						*C6713_I2C0_DXR_RG = OUT_DATA[transmit_words];
						r = T2E6XXX_OK;
						transmit_words++;
					}
				}
			}
			else
			{
				if (C6713_I2C0_STR_ICRRDY_IS_ON)
				{
					// In this case I2C0 module must receive data or error condition
					if (!receive_success)
					{
						// Receive ready condition: the data receive register is ready to be read by DSP
						// read received data word
						IN_DATA[received_words] = *C6713_I2C0_DRR_RG;
						// check if I2C0 module has received correct data
						if (IN_DATA[received_words] != OUT_DATA[received_words + 1])
							r = T2E6XXX_I2C0_RRDY_ERR;
						else
						{
							received_words++;
							if (received_words == DATA_LEN)
							{
								// If all data words have been received go to the end
								C6713_CLEAR_I2C0_STR_ICRRDY;
								receive_success = 1;
								r = T2E6XXX_OK;
								goto halt_ok;
							}
							else
							{
								C6713_CLEAR_I2C0_STR_ICRRDY;
								r = T2E6XXX_OK;
							}
						}
					}
					else
						// All data words have been already received!
						r = T2E6XXX_I2C0_RRDY_ERR;
				}
			}
		}
	}

	// get here on any error condition
	// variable 'r' contains returned error code
error:
	while(1);

	// get here in case transmit and receive data for the first pass is OK
halt_ok:
	if (first_pass == 0)
	{
		// clear all flags for the first pass and renew OUT_DATA array
		transmit_success = 0;
		receive_success = 0;
		transmit_words = 0;
		received_words = 0;
		first_pass = 1;
		for (i = 0; i < MSG_LEN; i++)
			OUT_DATA[i] = i;
		goto start;
	}
	else
	// get here in case transmit and receive data for the second pass is OK
		while(1);
}

