/****************************************************
	TORNADO-E2/6713 Demo Utility.
	I2C-0/1 ports test. rev.1A
	(C) MicroLAB Systems, 2006


	Description:
	------------
	This demo tests DSP I2C-0/1 ports at 400 kHz.
	This demo configures I2C-1 port as master and I2C-0 port as slave and transmits data array.
	User must connect both I2C ports on E2/6713 board and run this demo.


	TORNADO platform:
	-----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

	Copyright:
	----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.

		Note!!!:
		--------
		1. In order to run this demo both I2C ports on E2/6713 board must be installed!

	Board configuration and demo parameters:
	----------------------------------------
	1. On-board bootmode configuration switch: HPI
	2. Variable r contains possible error codes:
		(Note, that error codes
			from -1 to -9 are applied to common errors,
			from -10 to -19 are applied to C6xxx on-chip registers errors,
			from -20 to -29 are applied to DSP initialization errors,
			from -30 to -39 are applied to DSP ports configuration errors,
			from -100 to -109 are applied to FLASH PROGRAMMING CONTROL errors,
			from -50 to -99 are applied to user defined errors)
		 0	- no errors;
		-2	- invalid H/W error (not E2/6713 board);
		-3	- H/W error (error writing to the DSP external control registers, etc);
		-4	- incompatible H/W error (the setting is not allowed on this board);
		-10	- error writing to C6713B DSP regs;
		-20	- error during PLL configuration;
		-30	- ASIOX rev.D DCM is not installed;
		-31	- incompatible DSP ports configuration;
		-50...-54	- I2C0 port is not correctly operated;
		-55...-59	- I2C1 port is not correctly operated;
*****************************************************/

#define __T2E6XXX_SELECT_E6713__			1

#include "t2e6xxx_dsp.h"

#define T2E6XXX_I2C0_ERR			-50		// I2C0 port is not correctly operated
#define T2E6XXX_I2C0_AL_ERR			-51		// I2C0 port arbitratin lost error
#define T2E6XXX_I2C0_NACK_ERR		-52		// I2C0 port no-acknowledgement error
#define T2E6XXX_I2C0_ARDY_ERR		-53		// I2C0 port ARDY error
#define T2E6XXX_I2C0_RRDY_ERR		-54		// I2C0 port receiver error

#define T2E6XXX_I2C1_ERR			-55		// I2C1 port is not correctly operated
#define T2E6XXX_I2C1_AL_ERR			-56		// I2C1 port arbitratin lost error
#define T2E6XXX_I2C1_NACK_ERR		-57		// I2C1 port no-acknowledgement error
#define T2E6XXX_I2C1_ARDY_ERR		-58		// I2C1 port ARDY error
#define T2E6XXX_I2C1_RRDY_ERR		-59		// I2C1 port receiver error

#define DATA_LEN					8		// Number of data words to be transmitted

unsigned I2C1_DATA[DATA_LEN]	= { 0x01, 0xe9,
									0x3f, 0x55,
									0xaa, 0xe3,
									0x8c, 0xa1
									};

unsigned I2C0_DATA[DATA_LEN];

int receive_success = 0, received_words = 0, transmit_success = 0, transmit_words = 0, r;

// I2C1 ISR
interrupt void i2c1_isr()
{
	int int_code;
	int_code = C6713_get_i2c1_isr_rg();
	switch(int_code)
	{
		case C6713_I2C1_ISR_NONE:
			r = T2E6XXX_I2C1_ERR;
			break;

		case C6713_I2C1_ISR_AL:
			// Arbitration is lost: this situation is happened due to
			// incorrect configuration both I2C modules
			r = T2E6XXX_I2C1_AL_ERR;
			break;
		
		case C6713_I2C1_ISR_NACK:
			// No-acknowledgement condition: this module I2C1 did not received
			// acknowledgement from the slave-receiver I2C0
			r = T2E6XXX_I2C1_NACK_ERR;
			break;

		case C6713_I2C1_ISR_RAR:
			// Check if all data words have been transmitted
			if (transmit_words != DATA_LEN)
			{
				//Not all the data have been transmitted
				r = T2E6XXX_I2C1_ARDY_ERR;
				break;
			}
			else
			{
				// Set STOP condition and exit with OK!
				C6713_SET_I2C1_MDR_STP_ON;
				transmit_success = 1;
				r = T2E6XXX_OK;
				break;
			}

		case C6713_I2C1_ISR_RDR:
			// In this demo I2C1 module acts as master-transmitter only!
			r = T2E6XXX_I2C1_RRDY_ERR;
			break;

		case C6713_I2C1_ISR_XDR:
			// Transmit ready condition: the data transmit register is ready
			// to accept new data
			if (transmit_words == DATA_LEN)
			{
				// If all data words have been transmitted ARDY IRQ must be generated!
				r = T2E6XXX_I2C1_ERR;
				break;
			}
			else
			{
				C6713_CLEAR_I2C1_STR_ICXRDY;
				*C6713_I2C1_DXR_RG = I2C1_DATA[transmit_words];
				r = T2E6XXX_OK;
				transmit_words++;
				break;
			}

		default:
			r = T2E6XXX_I2C1_ERR;
			break;
	}
}


// I2C0 ISR
interrupt void i2c0_isr()
{
	int int_code;
	int_code = C6713_get_i2c0_isr_rg();
	switch(int_code)
	{
		case C6713_I2C0_ISR_NONE:
			r = T2E6XXX_I2C0_ERR;
			break;

		case C6713_I2C0_ISR_AL:
			// Arbitration is lost: this situation is happened due to
			// incorrect configuration both I2C modules
			r = T2E6XXX_I2C0_AL_ERR;
			break;
		
		case C6713_I2C0_ISR_NACK:
			// Only applicable when the I2C module is transmitter!
			r = T2E6XXX_I2C0_NACK_ERR;
			break;

		case C6713_I2C0_ISR_RAR:
			// Only applicable when the I2C module is in the master mode!
			r = T2E6XXX_I2C0_ARDY_ERR;
			break;

		case C6713_I2C0_ISR_RDR:
			// Receive ready condition: the data receive register is ready
			// to be read by DSP
			// read received data word
			I2C0_DATA[received_words] = *C6713_I2C0_DRR_RG;
			// check if I2C0 module has received correct data
			if (I2C0_DATA[received_words] != I2C1_DATA[received_words])
			{
				r = T2E6XXX_I2C0_RRDY_ERR;
				break;
			}
			else
			{
				received_words++;
				if (received_words == DATA_LEN)
				{
					receive_success = 1;
					r = T2E6XXX_OK;
					break;
				}
				else
				{
					C6713_CLEAR_I2C0_STR_ICRRDY;
					r = T2E6XXX_OK;
					break;
				}
			}

		case C6713_I2C0_ISR_XDR:
			// Only applicable when the I2C module is transmitter!
			r = T2E6XXX_I2C0_ERR;
			break;

		default:
			r = T2E6XXX_I2C0_ERR;
			break;
	}
}


void main()
{
	int i;
	
	// configure DSP PLL with SPREAD_SPECTRUM_CLK not used and initialize DSP
	if ((r = T2E6XXX_init_dsp(0)) != T2E6XXX_OK)
		goto error;

	// enable I2C1 port via DSP ports configuration function (I2C0 port is always enabled)
	if ((r = T2E6XXX_set_dsp_ports_cnf(T2E6713_DSP_PORTS_CNF_I2C1)) != T2E6XXX_OK)
		goto error;

	// disable IRQs
	C6XXX_SET_GINT_ENABLE_OFF;

	// First of all check PIDs of I2C ports
	if (!C6713_I2C0_PID1_IS_C6713)
	{
		r = T2E6XXX_INVALID_HW_ERR;
		goto error;
	}
	if (!C6713_I2C0_PID2_IS_I2C)
	{
		r = T2E6XXX_INVALID_HW_ERR;
		goto error;
	}
	if (!C6713_I2C1_PID1_IS_C6713)
	{
		r = T2E6XXX_INVALID_HW_ERR;
		goto error;
	}
	if (!C6713_I2C1_PID2_IS_I2C)
	{
		r = T2E6XXX_INVALID_HW_ERR;
		goto error;
	}
	// configure I2C1 port as master-transmitter and I2C0 port as slave-receiver
	// set module clock frequency to 150 MHz
	*C6713_I2C0_PSC_RG = 0;
	*C6713_I2C1_PSC_RG = 0;
	// enable all I2C interrupts
	C6713_set_i2c0_ier_rg(0x1f);
	C6713_set_i2c1_ier_rg(0x1f);
	// set I2C1 master SCL clock frequency to 400 kHz
	*C6713_I2C1_CLKL_RG = 0xb6;
	*C6713_I2C1_CLKH_RG = 0xb5;
	// configure I2C0 own address to 1000000 and I2C1 own address to 1000001
	*C6713_I2C0_OAR_RG = 0x40;
	*C6713_I2C1_OAR_RG = 0x41;
	// configure I2C0 slave address to 1000001 (this is I2C1 own address)
	// and I2C1 slave address to 1000000 (this is I2C0 own address)
	*C6713_I2C0_SAR_RG = 0x41;
	*C6713_I2C1_SAR_RG = 0x40;
	// configure data count register to DATA_LEN data words (this is a size of message)
	*C6713_I2C1_CNT_RG = DATA_LEN;
	// take I2C1 and I2C0 modules out of reset condition and configure them
	// as master-transmitter and slave-receiver respectively
	C6713_set_i2c1_mdr_rg(0x620);
	C6713_set_i2c0_mdr_rg(0x20);
	// wait a little in order to the BB bit will correctly reflect the state of I2C bus
	// (see C6713B_sil_err_20.pdf)
	for (i = 0; i < 300000; i++);

	// configure DSP_INT14/15 IRQ source selectors to use I2C0/I2C1 IRQ sources.
	C6XXX_SET_INTMUX(14,C6XXX_INTSEL_I2CINT0);
	C6XXX_SET_INTMUX(15,C6XXX_INTSEL_I2CINT1);

	// enable DSP_INT14/15, disable others
	C6XXX_SET_INT_OFF_ALL;
	C6XXX_SET_INT_ENABLE_ON(14);
	C6XXX_SET_INT_ENABLE_ON(15);

	C6XXX_SET_INT_OFF_ALL;
	C6XXX_SET_GINT_ENABLE_ON;	// set GIE bit in order to enable DSP interrupts

	// poll the BUS BUSY bit (BB) in the I2C1 Status register
	while (C6713_I2C1_STR_BB_IS_ON);
	// write first transmit data into I2C1 Data Transmit register
	*C6713_I2C1_DXR_RG = I2C1_DATA[transmit_words];
	transmit_words++;
	// set START bit for I2C0 module to monitor the I2C bus
	C6713_SET_I2C0_MDR_STT_ON;
	// set START condition for I2C1 module
	C6713_SET_I2C1_MDR_STT_ON;

	while (r == T2E6XXX_OK)
	{
		if (transmit_success == 1 && receive_success == 1)
			goto halt_ok;
	}

	// get here on any error condition
	// variable 'r' contains returned error code
error:
	while(1);

halt_ok:
	// TRAP here in case transmit and receive operations are OK
	while(1);
}

