/********************************************************************
	TORNADO-E2/6713 Demo Utility.
	PIOX IRQ test. rev.1A
	(C) MicroLAB Systems, 2007


	Description:
	------------
	This demo tests PIOX interrupt requests from Asynchronous section of PIOX2 I/F (PIOX_ASYNC_IRQs) and 
	from Synchronous section of PIOX2 I/F (PIOX_SYNC_IRQs)

	User can apply external PIOX interrupt request from signal generator, 
	etc sources and observe DSP response via GPIO-4..7 I/O pins at JP13
	on-board edge connector, which are configued as the OUTPUTS. GPIO-4..7
	outputs can be observed via oscilloscope.

	PIOX2 interrupt request selectors for C6713 DSP and the 
	corresponding GPIO output pins, which are used to display
	respond of DSP to each of external interrupts, are set as 
	following:
		IRQ source:			DSP IRQ:		GPIO output:
		-----------			--------		------------
		PIOX_ASYNC_IRQ-0	EXT_INT4		GPIO-4
		PIOX_ASYNC_IRQ-1	EXT_INT5		GPIO-5
		PIOX_SYNC_IRQ-0		EXT_INT6		GPIO-6
		PIOX_SYNC_IRQ-1		EXT_INT7		GPIO-7

	
		Note!!!:
		--------
		1. In order to test interrupt requests from both sections of PIOX2 I/F
		 on E2/6713 board PIOX2 I/F header must be installed and PX_DCM_INSTALLED signal (pin A5)
		 and PX_DCM_SX_EN signal (pin A6) must be connected to GND!
			If only asynchronous section of PIOX2 I/F is used (only PX_DCM_INSTALLED signal (pin A5) is
		connected to GND) EXT_INT6 and EXT_INT7 are disabled!
		2. This test does not perform testing of RTC and GPIO-0
		 interrupt requests. Use T2E6713_RGIRQ.out test for testing RTC and GPIO-0 IRQs.

	TORNADO platform:
	-----------------
	MIRAGE JTAG emulator and TI C6000 CCS v.2 tools are required to 
	recompile this demo and to upload code to the on-board DSP 
	environment.

	Copyright:
	----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


	Board configuration and demo parameters:
	----------------------------------------
	1. On-board BOOTMODE configuration switch: HPI
	2. Variable r contains possible error codes:
		(Note, that error codes
			from -1 to -9 are applied to common errors,
			from -10 to -19 are applied to C6xxx on-chip registers errors,
			from -20 to -29 are applied to DSP initialization errors,
			from -30 to -39 are applied to DSP ports configuration errors,
			from -100 to -109 are applied to FLASH PROGRAMMING CONTROL errors,
			from -50 to -99 are applied to user defined errors)
		 0	- no errors;
		-2	- invalid H/W error (not E2/6713 board);
		-3	- H/W error (error writing to the DSP external control registers, etc);
		-4	- incompatible H/W error (the setting is not allowed on this board);
		-10	- error writing to C6713B DSP regs;
		-20	- error during PLL configuration;
		-50	- PIOX DCM is not installed;
***********************************************************************/

#define __T2E6XXX_SELECT_E6713__			1

#include "t2e6xxx_dsp.h"


#define T2E6XXX_PIOX_DCM_INSTALLED_ERR		-50		// PIOX DCM is not installed


#pragma NMI_INTERRUPT(nmi_isr)
// NMI ISR must be disabled in this test.
void nmi_isr()
{
}


// EXT_INT4 ISR must toggle GPIO-4 pin
interrupt void xint4_isr()
{
	T2E6XXX_set_gpio_data_rg(T2E6XXX_get_gpio_data_rg() ^ 0x10); 
}


// EXT_INT5 ISR must toggle GPIO-5 pin
interrupt void xint5_isr()
{
	T2E6XXX_set_gpio_data_rg(T2E6XXX_get_gpio_data_rg() ^ 0x20); 
}


// EXT_INT6 ISR must toggle GPIO-6 pin
interrupt void xint6_isr()
{
	T2E6XXX_set_gpio_data_rg(T2E6XXX_get_gpio_data_rg() ^ 0x40); 
}


// EXT_INT7 ISR must toggle GPIO-7 pin.
interrupt void xint7_isr()
{
	T2E6XXX_set_gpio_data_rg(T2E6XXX_get_gpio_data_rg() ^ 0x80); 
}


void main()
{
	int r;

	// configure DSP PLL with SPREAD_SPECTRUM_CLK not used and initialize DSP
	if ((r = T2E6XXX_init_dsp(0)) != T2E6XXX_OK)
		goto error;

	// disable IRQs
	C6XXX_SET_GINT_ENABLE_OFF;

	// configure EXT_INT polarity upon the E2/6xxx board type
	T2E6XXX_INIT_EXT_INT_POLARITY;

	// configure GPIO pins as outputs and clear output data
	T2E6XXX_set_gpio_dir_rg(0xff);
	if (T2E6XXX_get_gpio_dir_rg() != 0xff)
	{
		r = T2E6XXX_HW_ERR;
		goto error;
	}
	T2E6XXX_set_gpio_data_rg(0);
	if (T2E6XXX_get_gpio_data_rg() != 0)
	{
		r = T2E6XXX_HW_ERR;
		goto error;
	}

	// Disable NMI selector.
	T2E6XXX_SET_EXT_INT_SEL_OFF(T2E6XXX_NMI_SEL);
	if (!T2E6XXX_EXT_INT_SEL_IS_OFF(T2E6XXX_NMI_SEL))
	{
		r = T2E6XXX_HW_ERR;
		goto error;
	}

	// check if PIOX DCM is installed
	// It's needed to connect PX_DCM_INSTALLED signal (pin A5) to GND
	// to make sure that PIOX DCM I/F header is installed
	// If PIOX DCM I/F header isn't installed all interrupts are disabled
	if (T2E6XXX_PIOX_DCM_IS_INSTALLED)
	{
		// check if PIOX DCM has both synchronous and asynchronous I/Fs
		// or asynchronous I/F only
		if (T2E6XXX_PIOX_DCM_TYPE_IS_ASYNC_SYNC)
		{
			// Configure EXT_INT4..7 IRQ source selectors to use PIOX_ASYNC_IRQs and PIOX_SYNC_IRQs IRQ sources.
			T2E6XXX_SET_EXT_INT_SEL_PIOX_ASYNC_IRQ0(T2E6XXX_EXT_INT4_SEL);	// configure EXT_INT4 to PIOX_ASYNC_IRQ0
			T2E6XXX_SET_EXT_INT_SEL_PIOX_ASYNC_IRQ1(T2E6XXX_EXT_INT5_SEL);	// configure EXT_INT5 to PIOX_ASYNC_IRQ1
			T2E6XXX_SET_EXT_INT_SEL_PIOX_SYNC_IRQ0(T2E6XXX_EXT_INT6_SEL);	// configure EXT_INT6 to PIOX_SYNC_IRQ0
			T2E6XXX_SET_EXT_INT_SEL_PIOX_SYNC_IRQ1(T2E6XXX_EXT_INT7_SEL);	// configure EXT_INT7 to PIOX_SYNC_IRQ1
			// enable EXT_INT4..7
			C6XXX_SET_INT_ENABLE_ON(4);
			C6XXX_SET_INT_ENABLE_ON(5);
			C6XXX_SET_INT_ENABLE_ON(6);
			C6XXX_SET_INT_ENABLE_ON(7);
		}
		else
		{
			// In case only asynchronous section is used EXT_INT6 and EXT_INT7 are disabled
			// Configure EXT_INT4..5 IRQ source selectors to use PIOX_ASYNC_IRQs IRQ sources.
			T2E6XXX_SET_EXT_INT_SEL_PIOX_ASYNC_IRQ0(T2E6XXX_EXT_INT4_SEL);	// configure EXT_INT4 to PIOX_ASYNC_IRQ0
			T2E6XXX_SET_EXT_INT_SEL_PIOX_ASYNC_IRQ1(T2E6XXX_EXT_INT5_SEL);	// configure EXT_INT5 to PIOX_ASYNC_IRQ1
			// enable EXT_INT4..5
			C6XXX_SET_INT_ENABLE_ON(4);
			C6XXX_SET_INT_ENABLE_ON(5);
		}
		C6XXX_SET_INT_OFF_ALL;
		C6XXX_SET_GINT_ENABLE_ON;		/* set GIE bit */
	}
	else
	{
		r = T2E6XXX_PIOX_DCM_INSTALLED_ERR;
		goto error;
	}

	while (1);

error:
	// trap here on any error condition
	// variable 'r' contains returned error code
	while (1);
}

