/********************************************************************
	TORNADO-E2/6713 Demo Utility.
	RTC/GPIO-0 IRQ test. rev.1A
	(C) MicroLAB Systems, 2007


	Description:
	------------
	This demo tests RTC WDT and GPIO-0 interrupt requests.

	RTC WDT interrupt request is programmed to toggle each 0.01 second
	(refer to DS1501 datasheet for more details). User can observe DSP response
	 to RTC_WDT interrupt request toggls via GPIO-6 I/O pin at JP13 on-board edge connector.

	User can apply external GPIO-0 interrupt request from signal generator, 
	etc sources and observe DSP response via GPIO-7 I/O pins at JP13
	on-board edge connector.
	
	DSP NMI interrupt is not used in this test.

		Notes:
		------
			1. This test does not test SIOX/ASIOX, PIOX, UART and USB interrupt requests.

	RTC/GPIO-0 interrupt request selectors and the 
	corresponding GPIO output pins, which are used to display
	respond of DSP to each of external interrupts, are set as 
	following:
		IRQ source:			DSP IRQ:		GPIO output:
		-----------			--------		------------
		RTC_WDT				EXT_INT6		GPIO-6
		GPIO-0				EXT_INT7		GPIO-7


	TORNADO platform:
	-----------------
	MIRAGE JTAG emulator and TI C6000 CCS v.2 tools are required to 
	recompile this demo and to upload code to the on-board DSP 
	environment.

	Copyright:
	----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


	Board configuration and demo parameters:
	----------------------------------------
	1. On-board BOOTMODE configuration switch: HPI
	2. Variable r contains possible error codes:
		(Note, that error codes
			from -1 to -9 are applied to common errors,
			from -10 to -19 are applied to C6xxx on-chip registers errors,
			from -20 to -29 are applied to DSP initialization errors,
			from -30 to -39 are applied to DSP ports configuration errors,
			from -100 to -109 are applied to FLASH PROGRAMMING CONTROL errors,
			from -50 to -99 are applied to user defined errors)
		 0	- no errors;
		-2	- invalid H/W error (not E2/6713 board);
		-3	- H/W error (error writing to the DSP external control registers, etc);
		-4	- incompatible H/W error (the setting is not allowed on this board);
		-10	- error writing to C6713B DSP regs;
		-20	- error during PLL configuration;
		-50..-53	- errors occurred during this test;
***********************************************************************/

#define __T2E6XXX_SELECT_E6713__			1

#include "t2e6xxx_dsp.h"

#define T2E6XXX_RTC1_CNTRA_RG_ERR			-50		// error setting RTC Control A register
#define T2E6XXX_RTC1_CNTRB_RG_ERR			-51		// error setting RTC Control B register
#define T2E6XXX_RTC1_CLK_RG_ERR				-52		// error setting RTC clock registers
#define T2E6XXX_RTC1_WDT_RG_ERR				-53		// error setting RTC WDT registers

#pragma NMI_INTERRUPT(nmi_isr)
// NMI ISR is not used in this test.
void nmi_isr(void)
{
}


// EXT_INT4 ISR is not used in this test
interrupt void xint4_isr()
{
}


// EXT_INT5 ISR is not used in this test
interrupt void xint5_isr()
{
}


// EXT_INT6 ISR must toggle GPIO-6 pin
// This ISR is linked to RTC WDT IRQ. In order to restart WDT,
// any of the RTC WDT registers shall be written and the
// RTC ontrol A register shall be read to clear IRQ flags
interrupt void xint6_isr()
{
	int d;
	T2E6XXX_set_rtc1_rg(T2E6XXX_RTC1_WDT_SECONDS_FRACTION_RG, 0x01);
	d = T2E6XXX_get_rtc1_rg(T2E6XXX_RTC1_CNTRA_RG);
	T2E6XXX_set_gpio_data_rg(T2E6XXX_get_gpio_data_rg() ^ 0x40); 
}


// EXT_INT7 ISR must toggle GPIO-7 pin.
interrupt void xint7_isr()
{
	T2E6XXX_set_gpio_data_rg(T2E6XXX_get_gpio_data_rg() ^ 0x80); 
}



void main()
{
	int j, r;

	// configure DSP PLL with SPREAD_SPECTRUM_CLK not used and initialize DSP
	if ((r = T2E6XXX_init_dsp(0)) != T2E6XXX_OK)
		goto error;

	// disable IRQs
	C6XXX_SET_GINT_ENABLE_OFF;

	// configure EXT_INT polarity upon the E2/6xxx board type
	T2E6XXX_INIT_EXT_INT_POLARITY;

	// configure GPIO pins as outputs (except GPIO-0)and clear output data
	T2E6XXX_set_gpio_dir_rg(0xfe);
	if (T2E6XXX_get_gpio_dir_rg() != 0xfe)
	{
		r = T2E6XXX_HW_ERR;
		goto error;
	}
	T2E6XXX_set_gpio_data_rg(0);
	// Note that GPIO-0 is configured as input and is in z-state
	if (T2E6XXX_get_gpio_data_rg() != 0x1)
	{
		r = T2E6XXX_HW_ERR;
		goto error;
	}

	// configure RTC WDT registers

	// enable access to RTC controller
	T2E6XXX_SET_RTC_EN_ON;
	if (!T2E6XXX_RTC_EN_IS_ON)
	{
		r = T2E6XXX_HW_ERR;
		goto error;
	}

	// clear TE bit of RTC ontrol B register and freeze RTC clock registers
	T2E6XXX_set_rtc1_rg(T2E6XXX_RTC1_CNTRB_RG, 0x0);
	if ((T2E6XXX_get_rtc1_rg(T2E6XXX_RTC1_CNTRB_RG)) != 0x0)
	{
		r = T2E6XXX_RTC1_CNTRB_RG_ERR;
		goto error;
	}
	// clear FLAG bits of RTC ontrol A register and set RTC_PWR pin
	// in the high-impendance state
	// Note that only reading of ontrol A register can clear all IRQ flags!
	j = T2E6XXX_get_rtc1_rg(T2E6XXX_RTC1_CNTRA_RG);
	T2E6XXX_set_rtc1_rg(T2E6XXX_RTC1_CNTRA_RG, 0x10);
	if ((T2E6XXX_get_rtc1_rg(T2E6XXX_RTC1_CNTRA_RG)) != 0x10)
	{
		r = T2E6XXX_RTC1_CNTRA_RG_ERR;
		goto error;
	}
	// enable RTC clock oscillator (EOSC=0)
	T2E6XXX_set_rtc1_rg(T2E6XXX_RTC1_CLK_MONTH_RG, 0x0);
	if (T2E6XXX_get_rtc1_rg(T2E6XXX_RTC1_CLK_MONTH_RG) != 0x0)
	{
		r = T2E6XXX_RTC1_CLK_RG_ERR;
		goto error;
	}

	// set RTC WDT registers to generate WDT each 0.01 sec
	T2E6XXX_set_rtc1_rg(T2E6XXX_RTC1_WDT_SECONDS_FRACTION_RG, 0x01);
	if (T2E6XXX_get_rtc1_rg(T2E6XXX_RTC1_WDT_SECONDS_FRACTION_RG) != 0x01)
	{
		r = T2E6XXX_RTC1_WDT_RG_ERR;
		goto error;
	}
	T2E6XXX_set_rtc1_rg(T2E6XXX_RTC1_WDT_SECONDS_RG, 0x0);
	if (T2E6XXX_get_rtc1_rg(T2E6XXX_RTC1_WDT_SECONDS_RG) != 0x0)
	{
		r = T2E6XXX_RTC1_WDT_RG_ERR;
		goto error;
	}
	// set TE bit of RTC ontrol B register and unfreeze RTC clock registers
	// Also enable Watchdog repetitive mode
	T2E6XXX_set_rtc1_rg(T2E6XXX_RTC1_CNTRB_RG, 0x82);
	if ((T2E6XXX_get_rtc1_rg(T2E6XXX_RTC1_CNTRB_RG)) != 0x82)
	{
		r = T2E6XXX_RTC1_CNTRB_RG_ERR;
		goto error;
	}

	// configure EXT_INT6/7 IRQ source selectors to use GPIO-0/RTC IRQ sources.
	// Disable NMI IRQ source selector.
	T2E6XXX_SET_EXT_INT_SEL_OFF(T2E6XXX_NMI_SEL);
	if (!T2E6XXX_EXT_INT_SEL_IS_OFF(T2E6XXX_NMI_SEL))
	{
		r = T2E6XXX_HW_ERR;
		goto error;
	}
	T2E6XXX_SET_EXT_INT_SEL_OFF(T2E6XXX_EXT_INT4_SEL);
	T2E6XXX_SET_EXT_INT_SEL_OFF(T2E6XXX_EXT_INT5_SEL);
	T2E6XXX_SET_EXT_INT_SEL_RTC(T2E6XXX_EXT_INT6_SEL);		// configure INT6 to RTC_WDT
	T2E6XXX_SET_EXT_INT_SEL_GPIO0(T2E6XXX_EXT_INT7_SEL);	// configure INT7 to GPIO-0

	// enable/disable external DSP interrupts via DSP on-chip IER register
	// enable EXT_INT6/7, disable EXT_INT4/5
	C6XXX_SET_INT_ENABLE_OFF(4);
	C6XXX_SET_INT_ENABLE_OFF(5);
	C6XXX_SET_INT_ENABLE_ON(6);
	C6XXX_SET_INT_ENABLE_ON(7);

	C6XXX_SET_INT_OFF_ALL;
	C6XXX_SET_GINT_ENABLE_ON;		/* set GIE bit */

	while(1);

error:
	// trap here on any error condition
	// variable 'r' contains returned error code
	while (1);
}

