/****************************************************
	TORNADO-E2/6713 Demo Utility.
	ASIOX rev.D r/w test for E2/6713 boards.
	(C) MicroLAB Systems, 2007

	Description:
	------------
	This demo checks for R/W buffers of ASIOX rev.D data bus
	of TORNADO-E2/6713 boards.

		!!!CAUTION!!!
			DO NOT APPLY ANY OSCILLOSCOPE NODES to the ASIOX DATA BUS!
			(written data must stay precharged on ASIOX data buffers).

		Note!!!:
		--------
		1. In order to run this demo on E2/6713 board
		 ASIOX rev.D I/F header must be installed, ASX_DCM_INSTALLED signal (pin 115),
		 ASX_DCM_PB_EN signal (pin 116) and ASX_READY signal (pin 73) must be connected to GND!

	External grounding jumper is required for ASX_READY input pin of
	ASIOX rev.D interface site connector, which must be installed into
	pin #73 of on-board JP6 connector in order to permanently confirm
	ASIOX rev.D parallel data bus cycles.

	GPIO-7 on-chip output is used as the progress indicator and is 
	toggling in case of correct test procurement.


	TORNADO platform:
	-----------------
	This demo does not utilize any host resources.
	TI CCS6000 tools are required to recompile this demo.
	MIRAGE JTAG emulator are required to 
	upload code to the on-board DSP environment.

	Copyright:
	----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


	Demo and Signal Parameters:
	---------------------------
	1. In order to use this demo on E2/6713 board
		ASX_DCM_INSTALLED signal (pin 115), ASX_DCM_PB_EN signal (pin 116)
		and ASX_READY signal (pin 73) must be connected to GND!
	2. On-board BOOTMODE configuration switch: HPI boot
	3. Variable r contains possible error codes:
		(Note, that error codes
			from -1 to -9 are applied to common errors,
			from -10 to -19 are applied to C6xxx on-chip registers errors,
			from -20 to -29 are applied to DSP initialization errors,
			from -30 to -39 are applied to DSP ports configuration errors,
			from -100 to -109 are applied to FLASH PROGRAMMING CONTROL errors,
			from -50 to -99 are applied to user defined errors)
		 0	- no errors;
		-2	- invalid H/W error (not E2/6713 board);
		-3	- H/W error (error writing to the DSP external control registers, etc);
		-4	- incompatible H/W error (the setting is not allowed on this board);
		-10	- error writing to C6713B DSP regs;
		-20	- error during PLL configuration;
		-50	- ASIOX rev.D I/F is not installed;
		-51	- Parallel bus for ASIOX rev.D DCM is not supported;
*****************************************************/

#define __T2E6XXX_SELECT_E6713__			1

#include "t2e6xxx_dsp.h"

#define T2E6XXX_ASIOX_DCM_IS_NOT_INSTALLED_ERR	-50		// ASIOX rev.D I/F is not installed
#define T2E6XXX_ASIOX_DCM_PIO_ERR				-51		// Parallel bus for ASIOX rev.D DCM is not supported

unsigned pass_counter = 0;

void main()
{
	unsigned d, i;
	unsigned *xptr;
	int r;

	// configure DSP PLL with SPREAD_SPECTRUM_CLK not used and initialize DSP
	if ((r = T2E6XXX_init_dsp(0)) != T2E6XXX_OK)
		goto error;

	// check if ASIOX rev.D DCM (or adapter boards) is installed
	// It's needed to connect ASX_DCM_INSTALLED signal (pin 115) to GND
	if (!T2E6XXX_ASIOX_DCM_IS_INSTALLED)
	{
		r = T2E6XXX_ASIOX_DCM_IS_NOT_INSTALLED_ERR;
		goto error;
	}
	// check if Parallel bus for ASIOX rev.D DCM is supported
	// It's needed to connect ASX_DCM_PB_EN signal (pin 116) to GND
	// to enable ASIOX address and data transceivers
	if (!T2E6XXX_ASIOX_DCM_TYPE_IS_SIO_PIO)
	{
		r = T2E6XXX_ASIOX_DCM_PIO_ERR;
		goto error;
	}

	// configure GPIO pins as outputs and clear output data
	T2E6XXX_set_gpio_dir_rg(0xff);
	if (T2E6XXX_get_gpio_dir_rg() != 0xff)
	{
		r = T2E6XXX_HW_ERR;
		goto error;
	}
	T2E6XXX_set_gpio_data_rg(0);
	if (T2E6XXX_get_gpio_data_rg() != 0)
	{
		r = T2E6XXX_HW_ERR;
		goto error;
	}

	// assign pointer for alternative DSP bus access
	xptr = (unsigned *) T2E6XXX_SBSRAM_AREA_SADDR;

	while(1)
	{
		for (d = 0x1, i = 0; i < 16 ; i++)
		{ 
			// write to ASIOX regs
			T2E6XXX_write_asiox_w16(0,d);

			// write to any other address in SBSRAM in order to discard 
			// the precharged data on the bus
			*xptr = 0x12345678;

			//  verify read-back data
			//  !! do not apply any oscilloscope probe to the ASX_D[] pins in order
			//     to keep ASX_D[] data unchanged for reads
			if (T2E6XXX_read_asiox_w16(0) != d)
			{
				r = T2E6XXX_HW_ERR;
				goto error;
			}
			d = (d << 1);
		}

		// use GPIO-7 output as the progress indicator
		T2E6XXX_set_gpio_data_rg(T2E6XXX_get_gpio_data_rg() ^ 0x80); 
		pass_counter++;
	}

error:
	// get here on error condition
	// variable 'r' contains returned error code
	while(1);
}

