/********************************************************************
	TORNADO-E2/6713 Demo Utility.
	UART channel-B demo in ASYNC mode. rev.1A
	(C) MicroLAB Systems, 2007


	Description:
	------------
	This demo tests channel-B of UART in ASYNC mode with RS232C
	external interface. 

	External 9600 baud ASYNC terminal with RS232C interface or 
	PC COM port running in terminal mode at 9600 bauds shall be 
	used with this demo.

	After transmission of greeting message to external terminal,
	demo enters echo mode, which performs transmission of 
	received character back to terminal.

	This demo uses UART-B IRQ interrupt request, which is routed to DSP
	EXT_INT7 external interrupt request and occurs on each reception of
	input character. ISR toggles GPIO-7 output.
	Variable pass_counter is used as an indicator of successful interrupt handling.

	TORNADO platform:
	-----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

	Copyright:
	----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


	Board configuration and demo parameters:
	----------------------------------------
	1. On-board BOOTMODE configuration switch: HPI boot
	2. PC COM port settings:
		- baud rate:		9600
		- word length:		8 bits
		- stop bits:		1
	3. Variable r contains possible error codes:
		(Note, that error codes
			from -1 to -9 are applied to common errors,
			from -10 to -19 are applied to C6xxx on-chip registers errors,
			from -20 to -29 are applied to DSP initialization errors,
			from -30 to -39 are applied to DSP ports configuration errors,
			from -100 to -109 are applied to FLASH PROGRAMMING CONTROL errors,
			from -50 to -99 are applied to user defined errors)
		 0	- no errors;
		-2	- invalid H/W error (not E2/6713 board);
		-3	- H/W error (error writing to the DSP external control registers, etc);
		-4	- incompatible H/W error (the setting is not allowed on this board);
		-10	- error writing to C6713B DSP regs;
		-20	- error during PLL configuration;
***********************************************************************/

#define __T2E6XXX_SELECT_E6713__			1

#include "t2e6xxx_dsp.h"


char welcome[]	= "Welcome to E2/6713 UART-B demo!\n\n>>";
char cts_low[]	= "<< RTS: CTS=LOW >>";
char cts_high[]	= "<< RTS: CTS=HIGH >>";
char dsr_low[]	= "<< DTR: DSR=LOW >>";
char dsr_high[]	= "<< DTR: DSR=HIGH >>";

// general UART register access macro defs for easy selection of UART-A/B channel
#define READ_UART_RG(r)			T2E6XXX_get_uartb_rg(r)
#define WRITE_UART_RG(r,d)		T2E6XXX_set_uartb_rg(r,d)

unsigned pass_counter = 0;

#pragma NMI_INTERRUPT(nmi_isr)
// NMI ISR must be disabled in this test.
void nmi_isr()
{
}


// EXT_INT4 IRQ handler
// (not used in this demo)
interrupt void xint4_isr()
{
}


// EXT_INT5 IRQ handler
// (not used in this demo)
interrupt void xint5_isr()
{
}


// EXT_INT6 IRQ handler
// (not used in this demo)
interrupt void xint6_isr()
{
}


// EXT_INT7 ISR must toggle GPIO-7 pin.
interrupt void xint7_isr()
{
	T2E6XXX_set_gpio_data_rg(T2E6XXX_get_gpio_data_rg() ^ 0x80); 
	pass_counter++;
}


void uart_transmit_char(int c)
// waits for UART transmitter is empty and transmits one byte
{
	while (!(READ_UART_RG(T2E6XXX_UART_LSTAT_RG) & T2E6XXX_UART_LSTAT_THEMPTY));
	WRITE_UART_RG(T2E6XXX_UART_TDATA_RG,c);
}

unsigned uart_receive_char()
// waits for UART receiver is ready and reads input byte
{
	while (!(READ_UART_RG(T2E6XXX_UART_LSTAT_RG) & T2E6XXX_UART_LSTAT_RRDY));
	return(READ_UART_RG(T2E6XXX_UART_RDATA_RG));
}

int uart_receiver_status()
// returns UART receiver status
{
	return(READ_UART_RG(T2E6XXX_UART_LSTAT_RG) & T2E6XXX_UART_LSTAT_RRDY);
}

void uart_transmit_mess(char *s)
// transmits a string until '0' charachter
{
	while((*s &0xff) != 0)
		uart_transmit_char(*(s++));
}


void main()
{
	int modem_status, r;

	// configure DSP PLL with SPREAD_SPECTRUM_CLK not used and initialize DSP
	if ((r = T2E6XXX_init_dsp(0)) != T2E6XXX_OK)
		goto error;

	// disable IRQs
	C6XXX_SET_GINT_ENABLE_OFF;

	WRITE_UART_RG(T2E6XXX_UART_SCRPAD_RG, 0x55);
	if (READ_UART_RG(T2E6XXX_UART_SCRPAD_RG) != 0x55)
	{
		r = T2E6XXX_HW_ERR;
		goto error;
	}

	// initializes UART to configure UART for 9600/8/N/1
	WRITE_UART_RG(T2E6XXX_UART_LCNTR_RG, T2E6XXX_UART_LCNTR_DLAB | T2E6XXX_UART_LCNTR_WLEN8 | T2E6XXX_UART_LCNTR_STOP1);
	WRITE_UART_RG(T2E6XXX_UART_DIVL_RG, T2E6XXX_UART_DIVL_9600);
	WRITE_UART_RG(T2E6XXX_UART_DIVH_RG, T2E6XXX_UART_DIVH_9600);
	WRITE_UART_RG(T2E6XXX_UART_LCNTR_RG, T2E6XXX_UART_LCNTR_WLEN8 | T2E6XXX_UART_LCNTR_STOP1);

	// enable UART interrupt output
	WRITE_UART_RG(T2E6XXX_UART_MCNTR_RG, T2E6XXX_UART_MCNTR_INT_OE_ON);

	// enable UART interrupt generation on receiver data ready condition
	WRITE_UART_RG(T2E6XXX_UART_IE_RG, T2E6XXX_UART_IE_RRDY);

	// configure EXT_INT polarity upon the E2/6xxx board type
	T2E6XXX_INIT_EXT_INT_POLARITY;

	// configure GPIO pins as outputs and clear output data
	T2E6XXX_set_gpio_dir_rg(0xff);
	if (T2E6XXX_get_gpio_dir_rg() != 0xff)
	{
		r = T2E6XXX_HW_ERR;
		goto error;
	}
	T2E6XXX_set_gpio_data_rg(0);
	if (T2E6XXX_get_gpio_data_rg() != 0)
	{
		r = T2E6XXX_HW_ERR;
		goto error;
	}

	// Configure EXT_INT7 IRQ source selector to use UART-B IRQ source.
	// Disable NMI selector.
	T2E6XXX_SET_EXT_INT_SEL_OFF(T2E6XXX_NMI_SEL);				// disable NMI
	if (!T2E6XXX_EXT_INT_SEL_IS_OFF(T2E6XXX_NMI_SEL))
	{
		r = T2E6XXX_HW_ERR;
		goto error;
	}
	T2E6XXX_SET_EXT_INT_SEL_OFF(T2E6XXX_EXT_INT4_SEL);		// disable EXT_INT4
	T2E6XXX_SET_EXT_INT_SEL_OFF(T2E6XXX_EXT_INT5_SEL);		// disable EXT_INT5
	T2E6XXX_SET_EXT_INT_SEL_OFF(T2E6XXX_EXT_INT6_SEL);		// disable EXT_INT6
	T2E6XXX_SET_EXT_INT_SEL_UARTB(T2E6XXX_EXT_INT7_SEL);	// configure EXT_INT7 to UART-B IRQ

	// enable/disable external DSP interrupts via DSP on-chip IER register
	// enable EXT_INT7, disable EXT_INT4..6
	C6XXX_SET_INT_ENABLE_OFF(4);
	C6XXX_SET_INT_ENABLE_OFF(5);
	C6XXX_SET_INT_ENABLE_OFF(6);
	C6XXX_SET_INT_ENABLE_ON(7);

	C6XXX_SET_INT_OFF_ALL;
	C6XXX_SET_GINT_ENABLE_ON;		/* set GIE bit */

	// transmit greeting message upon the board revision
	uart_transmit_mess(welcome);

	// enter infinite loop
	while (1)
	{
		// read and process modem status in order to process CTS/DSR inputs
		modem_status = READ_UART_RG(T2E6XXX_UART_MSTAT_RG);

		// copy CTS->RTS
		if (modem_status & T2E6XXX_UART_MSTAT_CTS)
			WRITE_UART_RG(T2E6XXX_UART_MCNTR_RG, READ_UART_RG(T2E6XXX_UART_MCNTR_RG) | T2E6XXX_UART_MCNTR_RTS);
		else
			WRITE_UART_RG(T2E6XXX_UART_MCNTR_RG, READ_UART_RG(T2E6XXX_UART_MCNTR_RG) & ~T2E6XXX_UART_MCNTR_RTS);

		// copy DSR->DTR
		if (modem_status & T2E6XXX_UART_MSTAT_DSR)
			WRITE_UART_RG(T2E6XXX_UART_MCNTR_RG, READ_UART_RG(T2E6XXX_UART_MCNTR_RG) | T2E6XXX_UART_MCNTR_DTR);
		else
			WRITE_UART_RG(T2E6XXX_UART_MCNTR_RG, READ_UART_RG(T2E6XXX_UART_MCNTR_RG) & ~T2E6XXX_UART_MCNTR_DTR);

		// detect CTS change and print the CTS change message
		if(modem_status & T2E6XXX_UART_MSTAT_DCTS)
		{
			if (modem_status & T2E6XXX_UART_MSTAT_CTS)
				uart_transmit_mess(cts_high);
			else
				uart_transmit_mess(cts_low);
		}

		// detect DSR change and print the DSR change message
		if (modem_status & T2E6XXX_UART_MSTAT_DDSR)
		{
			if (modem_status & T2E6XXX_UART_MSTAT_DSR)
				uart_transmit_mess(dsr_high);
			else
				uart_transmit_mess(dsr_low);
		}

		// check whether input data arrives and echos it back to the transmitter
		if (uart_receiver_status() == T2E6XXX_UART_LSTAT_RRDY)
			uart_transmit_char(uart_receive_char());
	}

error:
	// TRAP here in case of any error
	// variable 'r' contains returned error code
	while(1);
}

