/********************************************************************
	TORNADO-E2/6713 Demo Utility.
	USB controller demo. rev.1A
	(C) MicroLAB Systems, 2007


	Description:
	------------
	This demo scans the on-chip registers of the USB controller,
	 USB I/O signals, and generates the USB interrupt, which
	is routed to the DSP INT7 interrupt request input.
	GPIO-7 OUTPUT is toggling on each USB interrupt event.
	Variable pass_counter is used as an indicator of successful interrupt handling.

	This demo provides different tests for the USB1 (Agere USS820),
	 USB2 (PLX-NetChip NET2272) and USB3 (Philips ISP1761) controllers
	 because of only one of USB controllers may be installed at the instance.


	TORNADO platform:
	-----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

	Copyright:
	----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


	Board configuration and demo parameters:
	----------------------------------------
	1. On-board bootmode configuration switch: HPI
	2. Variable r contains possible error codes:
		(Note, that error codes
			from -1 to -9 are applied to common errors,
			from -10 to -19 are applied to C6xxx on-chip registers errors,
			from -20 to -29 are applied to DSP initialization errors,
			from -30 to -39 are applied to DSP ports configuration errors,
			from -100 to -109 are applied to FLASH PROGRAMMING CONTROL errors,
			from -50 to -99 are applied to user defined errors)
		 0	- no errors;
		-2	- invalid H/W error (not E2/6713 board);
		-3	- H/W error (error writing to the DSP external control registers, etc);
		-4	- incompatible H/W error (the setting is not allowed on this board);
		-10	- error writing to C6713B DSP regs;
		-20	- error during PLL configuration;
***********************************************************************/

#define __T2E6XXX_SELECT_E6713__			1

#include "t2e6xxx_dsp.h"

unsigned pass_counter = 0;

#pragma NMI_INTERRUPT(nmi_isr)
// NMI ISR must be disabled in this test.
void nmi_isr()
{
}


// interrupt handler for EXT_INT4
// (not used in this demo)
interrupt void xint4_isr()
{
}


// interrupt handler for EXT_INT5
// (not used in this demo)
interrupt void xint5_isr()
{
}

// interrupt handler for EXT_INT6
// (not used in this demo)
interrupt void xint6_isr()
{
}


// interrupt handler for EXT_INT7
// Occurs on simulated USB interrupt request. Toggles GPIO-7 output.
interrupt void xint7_isr()
{
	// toggle the GPIO-7 output pin
	T2E6XXX_set_gpio_data_rg(T2E6XXX_get_gpio_data_rg() ^ 0x80);

	// make specific actions for the USB2 controller
	if (T2E6XXX_USB_ID_IS_USB2)
	{
		// clear the VBUS status change detector in the USB2 INTSTAT0 register
		T2E6XXX_set_usb2_rg(T2E6XXX_USB2_IRQSTAT1_RG, 0x4);

		// set GPIO-6 as the copy of the VBUS signal status
		if (T2E6XXX_get_usb2_rg(T2E6XXX_USB2_USBCTL1_RG) & 0x1)
			T2E6XXX_SET_GPIO6_DATA_ON;
		else
			T2E6XXX_SET_GPIO6_DATA_OFF;
	}
	pass_counter++;
}


// USB write-to-read pause, which is required for
// operations as WRITE to SCR register, which initializes
// internal state machine operations and requires some
// time to execute
// C6713B DSP is very fast and immediate reads may return
// incorrect results.
void wait()
{
	int i;
	for (i = 0; i < 20; i++);
}


void main()
{
	int i, pass, usb_id, r, first_pass = 0;
	unsigned d;

	// configure DSP PLL with SPREAD_SPECTRUM_CLK not used and initialize DSP
	if ((r = T2E6XXX_init_dsp(0)) != T2E6XXX_OK)
		goto error;

	// disable IRQs
	C6XXX_SET_GINT_ENABLE_OFF;

	// configure EXT_INT polarity upon the E2/6xxx board type
	T2E6XXX_INIT_EXT_INT_POLARITY;

	// configure GPIO pins as outputs and clear output data
	T2E6XXX_set_gpio_dir_rg(0xff);
	if (T2E6XXX_get_gpio_dir_rg() != 0xff)
	{
		r = T2E6XXX_HW_ERR;
		goto error;
	}
	T2E6XXX_set_gpio_data_rg(0);
	if (T2E6XXX_get_gpio_data_rg() != 0)
	{
		r = T2E6XXX_HW_ERR;
		goto error;
	}

	// configure EXT_INT7 IRQ source selectors to use USB IRQ sources.
	// Disable all other IRQ source selectors.
	T2E6XXX_SET_EXT_INT_SEL_OFF(T2E6XXX_NMI_SEL);
	if (!T2E6XXX_EXT_INT_SEL_IS_OFF(T2E6XXX_NMI_SEL))
	{
		r = T2E6XXX_HW_ERR;
		goto error;
	}
	T2E6XXX_SET_EXT_INT_SEL_OFF(T2E6XXX_EXT_INT4_SEL);
	T2E6XXX_SET_EXT_INT_SEL_OFF(T2E6XXX_EXT_INT5_SEL);
	T2E6XXX_SET_EXT_INT_SEL_OFF(T2E6XXX_EXT_INT6_SEL);
	T2E6XXX_SET_EXT_INT_SEL_USB(T2E6XXX_EXT_INT7_SEL);	// configure INT7 to USB IRQ

	// enable/disable external DSP interrupts via DSP on-chip IER register
	// enable EXT_INT7, disable EXT_INT4..6
	C6XXX_SET_INT_ENABLE_OFF(4);
	C6XXX_SET_INT_ENABLE_OFF(5);
	C6XXX_SET_INT_ENABLE_OFF(6);
	C6XXX_SET_INT_ENABLE_ON(7);

	C6XXX_SET_INT_OFF_ALL;
	C6XXX_SET_GINT_ENABLE_ON;		/* set GIE bit */

	// start infinite loop
	pass = 0;
	usb_id = T2E6XXX_get_usb_id();
	while (1)
	{
		switch(usb_id)
		{
			// check if Agere USS820 USB 1.1 Device only controller (USB1 controller) is installed
			case T2E6XXX_USB_ID_USB1:
				// check USB1 data bus via running 1<< to the SBIE reg
				for (i = 0, d = 1; i <= 8; i++)
				{
					T2E6XXX_set_usb1_rg(T2E6XXX_USB1_SBIE_RG, i == 0 ? 0: d);
					// perform false bus data access
					T2E6XXX_set_usb1_rg(T2E6XXX_USB1_SBIE1_RG, 0xff);

					// read and verify data
					if (T2E6XXX_get_usb1_rg(T2E6XXX_USB1_SBIE_RG) != (i == 0 ? 0: d))
					{
						r = T2E6XXX_HW_ERR;
						goto error;
					}

					if (i > 0)
						d <<= 1;
				}

				// perform software reset of the USB1 controller
				T2E6XXX_set_usb1_rg(T2E6XXX_USB1_SCR_RG, 0x8);
				wait();
				T2E6XXX_set_usb1_rg(T2E6XXX_USB1_SCR_RG, 0);
				wait();
				// and verify the results of reset
				if (   	T2E6XXX_get_usb1_rg(T2E6XXX_USB1_TXCNTH_RG) != 0
					|| 	T2E6XXX_get_usb1_rg(T2E6XXX_USB1_TXCNTL_RG) != 0
					|| 	T2E6XXX_get_usb1_rg(T2E6XXX_USB1_SBIE_RG) != 0
					|| 	T2E6XXX_get_usb1_rg(T2E6XXX_USB1_SBIE1_RG) != 0
					|| 	T2E6XXX_get_usb1_rg(T2E6XXX_USB1_SCR_RG) != 0
					)
				{
					r = T2E6XXX_HW_ERR;
					goto error;
				}

				//--- now toggle USB interrupt line using INT polarity bit
				// This will result in DSP EXT_INT7 interrupt events and will toggle GPIO-7 output.
				T2E6XXX_set_usb1_rg(T2E6XXX_USB1_SCR_RG, 0x80);
				wait();
				if (T2E6XXX_get_usb1_rg(T2E6XXX_USB1_SCR_RG) != 0x80)
				{
					r = T2E6XXX_HW_ERR;
					goto error;
				}
				T2E6XXX_set_usb1_rg(T2E6XXX_USB1_SCR_RG, 0x00);
				wait();
				if (T2E6XXX_get_usb1_rg(T2E6XXX_USB1_SCR_RG) != 0x00)
				{
					r = T2E6XXX_HW_ERR;
					goto error;
				}

				// now write random data to different USB1 registers
				T2E6XXX_set_usb1_rg(T2E6XXX_USB1_TXCNTH_RG, 0);
				T2E6XXX_set_usb1_rg(T2E6XXX_USB1_TXCNTL_RG, 0x91);
				T2E6XXX_set_usb1_rg(T2E6XXX_USB1_SBIE_RG, 0x5a);
				T2E6XXX_set_usb1_rg(T2E6XXX_USB1_SBIE1_RG, 0xa5);
				T2E6XXX_set_usb1_rg(T2E6XXX_USB1_FADDR_RG, 0x47);
				T2E6XXX_set_usb1_rg(T2E6XXX_USB1_SCRATCH_RG, 0x6e);
				T2E6XXX_set_usb1_rg(T2E6XXX_USB1_SCR_RG, 0x92);

				if ( (pass%2) == 0)
					T2E6XXX_set_usb1_rg(T2E6XXX_USB1_MCSR_RG, 0x0);
				else
					T2E6XXX_set_usb1_rg(T2E6XXX_USB1_MCSR_RG, 0x1);

				// and verify the results of writes
				if (T2E6XXX_get_usb1_rg(T2E6XXX_USB1_TXCNTL_RG) != 0x91)
				{
					r = T2E6XXX_HW_ERR;
					goto error;
				}
				if (T2E6XXX_get_usb1_rg(T2E6XXX_USB1_SBIE_RG) != 0x5a)
				{
					r = T2E6XXX_HW_ERR;
					goto error;
				}
				if (T2E6XXX_get_usb1_rg(T2E6XXX_USB1_SBIE1_RG) != 0xa5)
				{
					r = T2E6XXX_HW_ERR;
					goto error;
				}
				if (T2E6XXX_get_usb1_rg(T2E6XXX_USB1_FADDR_RG) != 0x47)
				{
					r = T2E6XXX_HW_ERR;
					goto error;
				}
				if (T2E6XXX_get_usb1_rg(T2E6XXX_USB1_SCRATCH_RG) != 0x6e)
				{
					r = T2E6XXX_HW_ERR;
					goto error;
				}
				if (T2E6XXX_get_usb1_rg(T2E6XXX_USB1_SCR_RG) != 0x92)
				{
					r = T2E6XXX_HW_ERR;
					goto error;
				}
				break;

			// check if PLX-NetChip NET2272 USB 2.0 Device only controller (USB2 controller) is installed
			case T2E6XXX_USB_ID_USB2:
				// check USB2 data bus via running 1<< to the SCRATCH reg
				for (i = 0,d = 1; i <= 8; i++)
				{
					T2E6XXX_set_usb2_rg(T2E6XXX_USB2_SCRATCH_RG, i == 0 ? 0: d);
					// perform false bus data access
					T2E6XXX_set_usb2_rg(T2E6XXX_USB2_REGADDRPTR_RG, 0x7f);
					// read and verify data
					if (T2E6XXX_get_usb2_rg(T2E6XXX_USB2_SCRATCH_RG) != (i == 0 ? 0: d))
					{
						r = T2E6XXX_HW_ERR;
						goto error;
					}
					if (i > 0)
						d <<= 1;
				}

				// now verify the USB2 chip revision info via the CHIPREV2272 and CHIPREVLEGACY registers
				T2E6XXX_set_usb2_rg(T2E6XXX_USB2_REGADDRPTR_RG, T2E6XXX_USB2_CHIPREV2272_RG);
				if (T2E6XXX_get_usb2_rg(T2E6XXX_USB2_REGDATA_RG) != 0x11)
				{
					r = T2E6XXX_HW_ERR;
					goto error;
				}
				T2E6XXX_set_usb2_rg(T2E6XXX_USB2_REGADDRPTR_RG, T2E6XXX_USB2_CHIPREVLEGACY_RG);
				if (T2E6XXX_get_usb2_rg(T2E6XXX_USB2_REGDATA_RG) != 0x40)
				{
					r = T2E6XXX_HW_ERR;
					goto error;
				}

				// now write random data to different USB2 registers and verify the results
				T2E6XXX_set_usb2_rg(T2E6XXX_USB2_SCRATCH_RG, 0xaa);
				T2E6XXX_set_usb2_rg(T2E6XXX_USB2_REGADDRPTR_RG, 0x55);
				T2E6XXX_set_usb2_rg(T2E6XXX_USB2_PAGESEL_RG, (pass%4));

				if (T2E6XXX_get_usb2_rg(T2E6XXX_USB2_SCRATCH_RG) != 0xaa)
				{
					r = T2E6XXX_HW_ERR;
					goto error;
				}
				if (T2E6XXX_get_usb2_rg(T2E6XXX_USB2_REGADDRPTR_RG) != 0x55)
				{
					r = T2E6XXX_HW_ERR;
					goto error;
				}
				if (T2E6XXX_get_usb2_rg(T2E6XXX_USB2_PAGESEL_RG) != (pass%4))
				{
					r = T2E6XXX_HW_ERR;
					goto error;
				}

				if (first_pass == 0)
				{
					// For the 1st pass only:
					//	Clear all possible interrupt request sources via INTSTAT0/1 registers.
					//	Enable USB2 controller interrupt on change of the VBUS signal.
					//	After first write to these registers. plug/unplug of the USB host cable will result in generation of
					//	the DSP EXT_INT7 interrupt and toggle of the GPIO-7 bit.

					// clear all possible IRQ flags via INTSTAT0 register
					T2E6XXX_set_usb2_rg(T2E6XXX_USB2_IRQSTAT0_RG, 0xff);
					// clear all possible flags via INTSTAT1 register, however exclude bit #3, which
					//  puts device into the low-power sleep mode.
					T2E6XXX_set_usb2_rg(T2E6XXX_USB2_IRQSTAT1_RG, 0xf7);

					T2E6XXX_set_usb2_rg(T2E6XXX_USB2_REGADDRPTR_RG, T2E6XXX_USB2_IRQENB0_RG);
					T2E6XXX_set_usb2_rg(T2E6XXX_USB2_REGDATA_RG, 0);
					T2E6XXX_set_usb2_rg(T2E6XXX_USB2_REGADDRPTR_RG, T2E6XXX_USB2_IRQENB1_RG);
					T2E6XXX_set_usb2_rg(T2E6XXX_USB2_REGDATA_RG, 0x4);

					// activate the USB host connection status so the RPU could set to the '1' state
					T2E6XXX_set_usb2_rg(T2E6XXX_USB2_USBCTL0_RG, 0x8);

					first_pass = 1;
				}

				// perform diagnostic pattern generation in case USB host is connected
				if ((T2E6XXX_get_usb2_rg(T2E6XXX_USB2_USBCTL1_RG) & 0x1))
				{
					// now put USB2 controller into the diagnostic mode and have it to generate the TEST_PACKET test.
					// This will provide switching of the USB data lines and can be observed via the oscilloscope.
					// put transceiver into the FULL-SPEED (12 Mbit/s) mode for diagnostic purpose
					T2E6XXX_set_usb2_rg(T2E6XXX_USB2_REGADDRPTR_RG, T2E6XXX_USB2_XCVRDIAG_RG);
					T2E6XXX_set_usb2_rg(T2E6XXX_USB2_REGDATA_RG, 0x4);
					// enable Test_packet generation at the USB data pins
					T2E6XXX_set_usb2_rg(T2E6XXX_USB2_REGADDRPTR_RG, T2E6XXX_USB2_USBTEST_RG);
					T2E6XXX_set_usb2_rg(T2E6XXX_USB2_REGDATA_RG, 0x4);

					// wait for some reasonable time (about 1..2 seconds)in order the USB data test patterns at the
					// USB data lines could be observed
					for (i = 0; i < 100000000; i++);

					// now disable test and put USB2 controller back into the normal operation mode
					T2E6XXX_set_usb2_rg(T2E6XXX_USB2_REGADDRPTR_RG, T2E6XXX_USB2_USBTEST_RG);
					T2E6XXX_set_usb2_rg(T2E6XXX_USB2_REGDATA_RG, 0x0);
					T2E6XXX_set_usb2_rg(T2E6XXX_USB2_REGADDRPTR_RG, T2E6XXX_USB2_XCVRDIAG_RG);
					T2E6XXX_set_usb2_rg(T2E6XXX_USB2_REGDATA_RG, 0x0);
					// wait for some reasonable time (about 1..2 seconds)in order the USB data test patterns at the
					// USB data lines could disappear and the pause is generated
					for (i = 0; i < 100000000; i++);
				}
				break;
				
			// check if Philips ISP1761 USB 2.0 Host/Device controller (USB3 controller) is installed
			case T2E6XXX_USB_ID_USB3:
				r = T2E6XXX_INCOMPATIBLE_HW_ERR;
				goto error;
				
			// go to error if USB controller is not installed
			default:
				r = T2E6XXX_INCOMPATIBLE_HW_ERR;
				goto error;
		}
		pass++;
		pass %= 1000;
		// disable interrupts in order USB interrupt would not distort the GPIO control register
		// enable interrupts again
		C6XXX_SET_GINT_ENABLE_OFF;
		C6XXX_SET_GINT_ENABLE_ON;
	}

error:
	// trap here on any error condition
	// variable 'r' contains returned error code
	while (1);
}

