/********************************************************************
	TORNADO-E2/6713 Demo Utility.
	FLASH Programming (Burn-in) utility for FLASH boot test.
	(C) MicroLAB Systems, 2007


	Description:
	------------
	This demo utility is used to program (burn-in) user supplied data
	array into on-board FLASH memory.

		!!! IMPORTANT NOTE !!!
		~~~~~~~~~~~~~~~~~~~~~~
		The on-board FLASH WRITE ENABLE switch SW1-2 and FLASH 64Mx8 BOOT SECTOR WRITE PROTECT
		switch SW1-3 must be set to the ON and OFF states correspondingly in order to run this demo.

	Input FLASH burn-in data array must be in an include file in 32-bit HEX
	format. This array can be either data or valid on-board C6713B DSP application,
	which can be converted directly from CCS C6000 generated output.

		!!! IMPORTANT NOTE !!!
		~~~~~~~~~~~~~~~~~~~~~~
			In case E2/6713 on-board FLASH must contain FLASH BOOT code, then
			it must be kept in mind that TI C6713B DSP allow only 1kbyte of FLASH data
			to be transferred to DSP on-chip memory during FLASH BOOT procedure (refer to
			TI C6713B DSP documentation for more details).

	Once input data array or application, which has to be programmed to FLASH, has
	been changed, then this utility must be recompiled using TI C6000 Code Composer
	C-compiler and then re-loaded into DSP environment via JTAG emulator and re-run.


	TORNADO platform:
	-----------------
	This demo does not utilize any host TORNADO-PX/DDC4G resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

	Copyright:
	----------
	This demo as well as other supplied utilities are distributed free
	of charge and without any obligation from MicroLAB Systems.


	Demo and Signal Parameters:
	---------------------------
	1. On-board configuration switch:
		  - HPI boot mode
		  - FLASH WRITE ENABLE switch SW1-2 must be set to ON
		  - FLASH 64Mx8 BOOT SECTOR WRITE PROTECT switch SW1-3 must be set to OFF
	2. Variable r contains possible error codes:
		(Note, that error codes
			from -1 to -9 are applied to common errors,
			from -10 to -19 are applied to C6xxx on-chip registers errors,
			from -20 to -29 are applied to DSP initialization errors,
			from -30 to -39 are applied to DSP ports configuration errors,
			from -100 to -109 are applied to FLASH PROGRAMMING CONTROL errors,
			from -50 to -99 are applied to user defined errors)
		 0		- no errors;
		-2		- invalid H/W error (not E2/6713 board);
		-3		- H/W error (error writing to the DSP external control registers, etc);
		-4		- incompatible H/W error (the setting is not allowed on this board);
		-10		- error writing to C6713B DSP regs;
		-20		- error during PLL configuration;
		-100	- unsupported FLASH device;
		-101	- FLASH erase error;
		-102	- FLASH write error;
		-103	- FLASH contents does not allow to write this data;
		-104	- invalid FLASH read data during AUTO-SELECT;
		-105	- FLASH writes are disabled by on-board SWITCH SW1-2 or Host Controller;
		-106	- Writes to FLASH 64Mx8 BOOT SECTOR are disabled by on-board SWITCH or Host Controller;
***********************************************************************/

#define __T2E6XXX_SELECT_E6713__			1
#define __T2E6XXX_INCLUDE_FLASH_CONTROL__	1

#include "t2e6xxx_dsp.h"

//~~~~~ include FLASH burn-in data array (32-bit C-hex format) ~~~~
// this array will have default unsigned int LUI_DATA[] reference
#include "wdt_boot.ci"
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

// FLASH data array size definition
#define FLASH_ARRAY_LEN		sizeof(LUI_DATA)

void main()
{
	int i, r;
	unsigned flash_addr;
	unsigned char *flash_src_data_ptr;

	// configure DSP PLL with SPREAD_SPECTRUM_CLK not used and initialize DSP
	if ((r = T2E6XXX_init_dsp(0)) != T2E6XXX_OK)
		goto error;

	// enable FLASH writes via FLASH_WR_EN bit of DSP_FLASH_CNTR_RG
	T2E6XXX_SET_FLASH_WR_EN_ON;
	if (!T2E6XXX_FLASH_WR_EN_IS_ON)
	{
		if (T2E6XXX_FLASH_XWRPROT_STAT_IS_ON)
			r = T2E6XXX_FLASH_XWRPROT_ERR;	// FLASH writes are disabled externally!
		else
			r = T2E6XXX_HW_ERR;	// H/W error

		goto error;
	}

	// enable writes to FLASH 64Mx8 BOOT SECTOR via DSP_FLASH_CNTR_RG
	// use this only in case FLASH 64Mx8 chip is installed!!!
	if (T2E6XXX_FLASH_LEN_IS_64MX8)
	{
		T2E6XXX_SET_FLASH64M_BOOT_SECTOR_WR_EN_ON;
		if (!T2E6XXX_FLASH64M_BOOT_SECTOR_WR_EN_IS_ON)
		{
			if (T2E6XXX_FLASH64M_BOOT_SECTOR_XWRPROT_STAT_IS_ON)
				r = T2E6XXX_FLASH64M_BOOT_SECTOR_XWRPROT_ERR;	// Writes to FLASH 64Mx8 BOOT SECTOR are disabled externally!
			else
				r = T2E6XXX_HW_ERR;	// H/W error

			goto error;
		}
	}

	// erase entire chip with verification
	if ((r = T2E6XXX_erase_flash_chip(T2E6XXX_ON)) != T2E6XXX_OK)
		goto error;

	// now program input array to FLASH
	// Note, that input data array must be transferred to FLASH memory in byte increment
	for (flash_addr = 0, i = 0, flash_src_data_ptr = (unsigned char*)LUI_DATA;
		i < FLASH_ARRAY_LEN;
		i++, flash_addr++, flash_src_data_ptr++)
	{
		if ((r = T2E6XXX_write_flash(flash_addr,*flash_src_data_ptr)) != T2E6XXX_OK)
			goto error;
	}

	// now verify written data
	// Note, that input data array must be transferred to FLASH memory in byte increment
	for (flash_addr = 0, i = 0, flash_src_data_ptr = (unsigned char*)LUI_DATA;
		i < FLASH_ARRAY_LEN;
		i++, flash_addr++, flash_src_data_ptr++)
	{
		if ((r = T2E6XXX_read_flash(flash_addr)) != *flash_src_data_ptr)
			goto error;
	}

term_ok:
	// TRAP here in case programming & verification is OK
	while(1);

error:
	// TRAP here in case of any FLASH API error
	// 'r' var will contain error code
	while(1);
}

