/********************************************************************
	TORNADO-E2/6713 Demo Utility.
	RTC (real-time clock) controller KICKSTART READ demo. rev.1A
	(C) MicroLAB Systems, 2007


	Description:
	------------
	This demo reads and verifies RTC control registers after KICKSTART or WAKEUP procedure has occurred.
	This demo must be used only after RTC_KICKSTART_WR demo has been run successfully,
	which should have initialized RTC and written RTC control registers.
	User must connect E2/6713 KICKSTART test board to JP16 connector, switch on MANUAL SWITCH (yellow LED must be ON),
	run this demo, then switch off MANUAL SWITCH (yellow LED must be OFF), the power of E2/6713 board
	will switched off, press KICKSTART button to start KICKSTART procedure or
	wait until WAKEUP will occurred (about 2 minutes).
	Red LED must be ON if RTC Remote Power Supply Control Circuit is functioned correctly!
	After power supply will be stabilized at its normal level user will have to run this demo.

	GPIO-6/7 outputs are used to indicate which procedure has been occurred (KICKSTART or WAKEUP):
		if KICKSTART has occurred then GPIO-7 output will be ON,
		if WAKEUP has occurred then GPIO-6 output will be ON!

	TORNADO platform:
	-----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

	Copyright:
	----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.

	Board configuration and demo parameters:
	----------------------------------------
	1. On-board bootmode configuration switch: HPI
	2. Variable r contains possible error codes:
		(Note, that error codes
			from -1 to -9 are applied to common errors,
			from -10 to -19 are applied to C6xxx on-chip registers errors,
			from -20 to -29 are applied to DSP initialization errors,
			from -30 to -39 are applied to DSP ports configuration errors,
			from -100 to -109 are applied to FLASH PROGRAMMING CONTROL errors,
			from -50 to -99 are applied to user defined errors)
		 0	- no errors;
		-2	- invalid H/W error (not E2/6713 board);
		-3	- H/W error (error writing to the DSP external control registers, etc);
		-4	- incompatible H/W error (the setting is not allowed on this board);
		-10	- error writing to C6713B DSP regs;
		-20	- error during PLL configuration;
		-50..-51	- errors occurred during this test;
***********************************************************************/

#define __T2E6XXX_SELECT_E6713__			1

#include "t2e6xxx_dsp.h"

#define T2E6XXX_PWR_ON_ERR					-50		// error during system power ON
#define T2E6XXX_RTC1_CNTRA_RG_ERR			-51		// error setting RTC Control A register

int r, d = 0, isr_count = 0;

#pragma NMI_INTERRUPT(nmi_isr)
// NMI ISR is not used in this test.
void nmi_isr(void)
{
}


// EXT_INT4 ISR
interrupt void xint4_isr()
{
}


// EXT_INT5
interrupt void xint5_isr()
{
}


// EXT_INT6 ISR
interrupt void xint6_isr()
{
}


// EXT_INT7 ISR must set GPIO-6 (WAKEUP) or GPIO-7 (KICKSTART) pins.
// This ISR is linked to RTC IRQ.
interrupt void xint7_isr()
{
	d = T2E6XXX_get_rtc1_rg(T2E6XXX_RTC1_CNTRA_RG);
	if ((d & 0x5) == 0x5)
		// KICKSTART has been occurred
		T2E6XXX_SET_GPIO7_DATA_ON;
	else
	{
		if ((d & 0x9) == 0x9)
			// WAKEUP has been occurred
			T2E6XXX_SET_GPIO6_DATA_ON;
		else
			// Error has been occurred during power up
			r = T2E6XXX_PWR_ON_ERR;
	}
	// clear all Interrupt Enable bits of RTC ontrol B register
	T2E6XXX_set_rtc1_rg(T2E6XXX_RTC1_CNTRB_RG, 0x80);
	isr_count = 1;
}



void main()
{
	// configure DSP PLL with SPREAD_SPECTRUM_CLK not used and initialize DSP
	if ((r = T2E6XXX_init_dsp(0)) != T2E6XXX_OK)
		goto error;

	// disable IRQs
	C6XXX_SET_GINT_ENABLE_OFF;

	// configure EXT_INT polarity upon the E2/6xxx board type
	T2E6XXX_INIT_EXT_INT_POLARITY;

	// configure GPIO pins as outputs and clear output data
	T2E6XXX_set_gpio_dir_rg(0xff);
	if (T2E6XXX_get_gpio_dir_rg() != 0xff)
	{
		r = T2E6XXX_HW_ERR;
		goto error;
	}
	T2E6XXX_set_gpio_data_rg(0);
	if (T2E6XXX_get_gpio_data_rg() != 0)
	{
		r = T2E6XXX_HW_ERR;
		goto error;
	}

	// enable access to RTC controller
	T2E6XXX_SET_RTC_EN_ON;
	if (!T2E6XXX_RTC_EN_IS_ON)
	{
		r = T2E6XXX_HW_ERR;
		goto error;
	}

	// configure EXT_INT7 IRQ source selectors to use RTC IRQ sources.
	// Disable NMI IRQ source selector.
	T2E6XXX_SET_EXT_INT_SEL_OFF(T2E6XXX_NMI_SEL);
	if (!T2E6XXX_EXT_INT_SEL_IS_OFF(T2E6XXX_NMI_SEL))
	{
		r = T2E6XXX_HW_ERR;
		goto error;
	}
	T2E6XXX_SET_EXT_INT_SEL_OFF(T2E6XXX_EXT_INT4_SEL);
	T2E6XXX_SET_EXT_INT_SEL_OFF(T2E6XXX_EXT_INT5_SEL);
	T2E6XXX_SET_EXT_INT_SEL_OFF(T2E6XXX_EXT_INT6_SEL);
	T2E6XXX_SET_EXT_INT_SEL_RTC(T2E6XXX_EXT_INT7_SEL);	// configure INT7 to RTC IRQ

	// enable/disable external DSP interrupts via DSP on-chip IER register
	// enable EXT_INT7, disable EXT_INT4..6
	C6XXX_SET_INT_ENABLE_OFF(4);
	C6XXX_SET_INT_ENABLE_OFF(5);
	C6XXX_SET_INT_ENABLE_OFF(6);
	C6XXX_SET_INT_ENABLE_ON(7);

	C6XXX_SET_INT_OFF_ALL;
	C6XXX_SET_GINT_ENABLE_ON;		/* set GIE bit */

	// wait here for RTC interrupt event
	while (r == T2E6XXX_OK)
	{
		if (isr_count == 1)
			goto halt_ok;
	}

error:
	// trap here on any error condition
	// variable 'r' contains returned error code

	// finally, disable access to RTC controller
	T2E6XXX_SET_RTC_EN_OFF;
	while (1);


halt_ok:
	// TRAP here in case KICKSTART or WAKEUP have been occurred
	// check state of GPIO-6 (WAKEUP) and GPIO-7 (KICKSTART) pins
	// to determine what event has been occurred

	// clear FLAG bits of RTC ontrol A register
	T2E6XXX_set_rtc1_rg(T2E6XXX_RTC1_CNTRA_RG, 0x0);
	if((T2E6XXX_get_rtc1_rg(T2E6XXX_RTC1_CNTRA_RG)) != 0x0)
	{
		r = T2E6XXX_RTC1_CNTRA_RG_ERR;
		goto error;
	}
	// disable IRQs
	C6XXX_SET_GINT_ENABLE_OFF;
	// finally, disable access to RTC controller
	T2E6XXX_SET_RTC_EN_OFF;
	while(1);
}

